module toolBox
  !-------------------------------------------------------------------------------
  interface getPrjDir
    module procedure getPrjDir
  end interface
contains
!-------------------------------------------------------------------------------
subroutine gauss(a,b,x,n)
! @JR: This subroutine has been downloaded from http://ww2.odu.edu/~agodunov.
! Credits: Alexander L Godunov, Department of Physics, Old Dominion University, Norfolk, VA

!===========================================================
! Solutions to a system of linear equations A*x=b
! Method: Gauss elimination (with scaling and pivoting)
! Alex G. (November 2009)
!-----------------------------------------------------------
! input ...
! a(n,n) - array of coefficients for matrix A
! b(n)   - array of the right hand coefficients b
! n      - number of equations (size of matrix A)
! output ...
! x(n)   - solutions
! comments ...
! the original arrays a(n,n) and b(n) will be destroyed
! during the calculation
!===========================================================
  implicit none
  integer n
  double precision a(n,n), b(n), x(n)
  double precision s(n)
  double precision c, pivot, store
  integer i, j, k, l
  ! step 1: begin forward elimination
  do k=1, n-1

   ! step 2: "scaling"
   ! s(i) will have the largest element from row i
     do i=k,n                       ! loop over rows
       s(i) = 0.0
       do j=k,n                    ! loop over elements of row i
         s(i) = max(s(i),abs(a(i,j)))
       end do
     end do

     ! step 3: "pivoting 1"
     ! find a row with the largest pivoting element
     pivot = abs(a(k,k)/s(k))
     l = k
     do j=k+1,n
       if(abs(a(j,k)/s(j)) > pivot) then
         pivot = abs(a(j,k)/s(j))
         l = j
       end if
     end do

   ! Check if the system has a singular matrix
     if(pivot == 0.0) then
       write(*,*) ' The matrix is singular '
       return
     end if

   ! step 4: "pivoting 2" interchange rows k and l (if needed)
   if (l /= k) then
     do j=k,n
        store = a(k,j)
        a(k,j) = a(l,j)
        a(l,j) = store
     end do
     store = b(k)
     b(k) = b(l)
     b(l) = store
   end if

   ! step 5: the elimination (after scaling and pivoting)
      do i=k+1,n
         c=a(i,k)/a(k,k)
         a(i,k) = 0.0
         b(i)=b(i)- c*b(k)
         do j=k+1,n
            a(i,j) = a(i,j)-c*a(k,j)
         end do
      end do
   end do

   ! step 6: back substiturion
   x(n) = b(n)/a(n,n)
   do i=n-1,1,-1
      c=0.0
      do j=i+1,n
        c= c + a(i,j)*x(j)
      end do
      x(i) = (b(i)- c)/a(i,i)
   end do
end subroutine gauss
!-------------------------------------------------------------------------------
! function checkv() result(u)
!   ! The purpose of this function is to check if statevariables vector is NaN
!   use vars
!   Integer i
!   logical u
!   double precision I1, sigmat, a
!
!   a = 25.0d0
!   sigmat = -a
!
!   do i = 1,size(v)
!     if (isNan(v(i))) then
!       u = .true.
!     end if
!   end do
!
!   if (v(1).le.sigmat) then
!     u = .true.
!   end if
!
!   if (v(2).le.sigmat) then
!     u = .true.
!   end if
!
!   if (v(3).le.sigmat) then
!     u = .true.
!   end if
!
!   end function
!-------------------------------------------------------------------------------
Subroutine getPrjDir( iPrjDir, iPrjLen, PrjDir)
  !DEC$ ATTRIBUTES NO_ARG_CHECK :: iPrjDir
      Implicit None
      Integer, intent(in) :: iPrjLen, iPrjDir(*)
      Character(len=255)  :: PrjDir
      Integer             :: i, ios

      PrjDir = ' '

      Do i=1,(iPrjLen-18)
        PrjDir(i:i) = Char( iPrjDir(i) )
      End Do
End Subroutine getPrjDir
!-------------------------------------------------------------------------------
Subroutine readBinaryStVars( PrjDir, fileName, length_unf )
    USE precision
    IMPLICIT NONE
    CHARACTER(len=*)      :: fileName, PrjDir
    CHARACTEr(len=255)    :: binName, gpSizeName
    CHARACTER(len=80)     :: Header
    CHARACTER(len=12)     :: BkHead = ' '
    CHARACTER(len=1)      :: typeCharacter
    INTEGER               :: NByts, nVals, j, iNumber, length_unf
    REAL(kind=realDBL)    :: rNumber, stateVar, output
    !---------------------------------------------------------------------------
    fileName = trim(PrjDir)//trim(fileName)
    binName  = trim(PrjDir)//'stVars.bin'
    ! gpSizeName = trim(PrjDir)//'gaussPoints.txt'

    INQUIRE (IOLENGTH=length_unf) output

          OPEN(UNIT=10, FILE=fileName, ACTION='READ', &
          ACCESS='STREAM', FORM='UNFORMATTED')

          OPEN(UNIT=11, FILE=binName, ACTION='WRITE', &
          ACCESS='DIRECT', FORM='UNFORMATTED', RECL=length_unf)

          ! Open(UNIT=12, Status='Replace', ACTION='WRITE', File=gpSizeName)
    !---------------------------------------------------------------------------
    !---------------------------------------------------------------------------
      READ(10)  Header
    !---------------------------------------------------------------------------
    ! Get stVars data
    !---------------------------------------------------------------------------
    DO WHILE (BkHead /='$$ENDOFFII##')
    READ(10)   BkHead,NByts
    typeCharacter = BkHead(10:10)
        IF (typeCharacter == 'I') THEN
          nVals = NByts/4
            DO j = 1, nVals
              READ(10)    iNumber
            END DO
        ELSE IF (typeCharacter == 'R') THEN
          nVals = NByts/8
            DO j = 1, nVals
              IF ( BkHead=='$$STVARALR##') THEN
                IF (j==1) THEN
                  ! WRITE(12,*) nVals
                END IF
                READ(10)         stateVar
                WRITE(11,REC=j)  stateVar
              ELSE
                READ(10)    rNumber
              END IF
            END DO
        END IF
    END DO
end subroutine readBinaryStVars
!-------------------------------------------------------------------------------
Subroutine storePrevStateVarsToFile( PrjDir )
  IMPLICIT NONE
  CHARACTER(len=255) :: PrjDir, nextFile, fileName, reclFile
  INTEGER            :: GP, length_unf

  Call getGPvalue( PrjDir, GP ) ! Get this points GP nr.
  IF (GP == 1) THEN             ! Store statevariables in a new binary file
     nextFile = trim(PrjDir)//'lastPhase.txt'
     reclFile = trim(PrjDir)//'RECL.txt'
     Open(131, Status='Old', File=nextFile)
     Read(131,*) fileName
     CALL readBinaryStVars( PrjDir, fileName, length_unf )
     Close(131)
     ! Store record length
     Open(135, Status='Replace', File=reclFile)
     Write(135,'(I5)') length_unf
     Close(135)
  end if
end Subroutine storePrevStateVarsToFile
!-------------------------------------------------------------------------------
Subroutine getGPvalue ( PrjDir, GP )
  IMPLICIT NONE
  CHARACTER(LEN=255) :: PrjDir, nextPoint, GaussP
  INTEGER            :: GP

  nextPoint = trim(PrjDir)//'gp.txt'
  Open(132, Status='Old', File=nextPoint)
  Read(132,*) GaussP
  read(GaussP,'(I)') GP  ! Convert character to integer
  Close(132)
End subroutine getGPvalue
!-------------------------------------------------------------------------------
Subroutine updateGPtracker (PrjDir, StVarPrevLen )
  ! Update gausspoint counter file
  IMPLICIT NONE
  CHARACTER(len=255) :: PrjDir, nextPoint, gpSizeName
  INTEGER            :: GP, gpSize, StVarPrevLen

  Call getGPvalue( PrjDir, GP )
  nextPoint = trim(PrjDir)//'gp.txt'
  ! gpSizeName = trim(PrjDir)//'gaussPoints.txt'

  ! Open(UNIT=127, Status='OLD', ACTION='READ', File=gpSizeName)
  ! READ(127,*) gpSize
  ! Close(127)

  Open(UNIT=134, Status='Replace', File=nextPoint)
    ! if (GP>=gpSize/StVarPrevLen) then
    !   Write(134,'(I5)') 1
    ! Else
      Write(134,'(I5)') GP+1
    ! END IF
  Close(134)
End subroutine updateGPtracker
!-------------------------------------------------------------------------------
Subroutine readStVarInGP ( PrjDir, StVarPrevLen, StVarPrev, choice)
  USE Precision
  IMPLICIT NONE
  INTEGER :: StVarPrevLen, GP, length_unf, i, choice
  CHARACTER(LEN=255) :: PrjDir, binName, reclFile, nextPoint
  REAL(kind=realDBL), DIMENSION(StVarPrevLen)  ::  StVarPrev
  if (choice==1) then
    binName = trim(PrjDir)//'stVars.bin'
  end if
  reclFile = trim(PrjDir)//'RECL.txt'

  Call getGPvalue( PrjDir, GP )
  Open(136, Status='Old', File=reclFile)
  Read(136,*) length_unf
  close(136)

  ! Read previous statevariables in same point
  OPEN(UNIT=133, FILE=binName, ACTION='READ', &
  ACCESS='DIRECT', FORM='UNFORMATTED', RECL=length_unf)
    DO i = (1+StVarPrevLen*(GP-1)),(StVarPrevLen*GP)
      READ(133,REC=i) StVarPrev(i-StVarPrevLen*(GP-1)) ! Correct i so that it starts at 1.
    end do
  Close(133)
end subroutine readStVarInGP
!-------------------------------------------------------------------------------
subroutine getInitialStructure( PrjDir, chi00 )
  USE Precision
  IMPLICIT NONE
  CHARACTER(len=255) :: fileName, PrjDir
  REAL(kind=realDBL) :: chi00
  fileName = trim(PrjDir)//'avgInput.txt'

  OPEN(UNIT=134, FILE=fileName, ACTION='READ', FORM='FORMATTED')
    read(134,'(D8.1)')  chi00
  Close(134)
end subroutine getInitialStructure
!-------------------------------------------------------------------------------
Subroutine checkIfCyclingCalculate ( PrjDir, existence )
  IMPLICIT NONE
  CHARACTER(len=255) :: PrjDir, nextFile
  LOGICAL            :: existence

  nextFile = trim(PrjDir)//'lastphase.txt'
  INQUIRE(FILE=nextFile, EXIST = existence)
end Subroutine checkIfCyclingCalculate
!-------------------------------------------------------------------------------
end module toolBox
