function createsymbolics()
% Modified Cam-Clay with destructuration
% @JR,   04.05.16
% @Jomy, 16.05.17

%% Model name:
ModelName = 'CamClay';

%% Material parameters, etc.
% Stiffness
k     = sym('k', 'real');
l     = sym('l', 'real');
nu    = sym('nu', 'real');

% Strength
phi   = sym('phi', 'real');
OCR   = sym('OCR', 'real');

% Intrinsic surface
pm   = sym('pm', 'real');
pm0  = sym('pm0', 'real');

%% Include common symbolics
commonsyms;
p = p + 0.1; % Avoid numerical instability when p = 0.
% Stiffness
K = p/k;
G = 3*(1-2*nu)*K/2/(1+nu);
isotropicElastic

%% Matrix properties
NonSym      =   0;   % 1 for non-symmetric D-matrix
iStrsDep    =   0;   % 1 for stress dependent D-matrix
iTang       =   0;   % 1 for tanget D-matrix
iTimeDep    =   0;   % 1 for time dependent D-matrix
includeStateVars=   1;   % 1 for include statevariables in D-matrix
matrixProp  = [NonSym; iStrsDep; iTang; iTimeDep; includeStateVars];

%% Input and state vectors
% Input parameters
c = [phi;l;k;nu;OCR];

% State variables
v  = [Sig ;lam ;pm ];
v0 = [Sig0;lam0;pm0];

%% Failure criteria
% Drucker-Prager failure criterion
phi2 = phi*pi/180;
M = 6*sin(phi2)/(3-sin(phi2));

% Yield function 
F = q^2 - M^2*p*(pm-p);
peq = solve(F==0,pm);

% Associated flow
Q = F;

%% Differentials
dEps_p    = diffvec(Q,Sig);
dEpsVol_p = diffvec(Q,s11) + diffvec(Q,s22) + diffvec(Q,s33);
dpm_dl   = 1/(l-k)*pm*dEpsVol_p;

%% Residual equations
r1 = Sig - Sig0 - D*(dEps - dEps_p*(lam-lam0));
r2 = pm  - pm0  - dpm_dl*(lam-lam0);
r3 = F;
Res = [r1;r2;r3];

% Jacobian matrix
Jacobis = jacobian(Res,v);

% Trial stress
sigTrial = Sig0 + D*dEps;

% Initialization of state variables
v00 = [Sig; 0; peq*OCR];


%% Output Fortran subroutines (for PLAXIS)
tic
%Output
folder = ['..\models\', ModelName];
mkdir(folder);
    
disp('Generating Fortran file...');
% %delete('residuals.f', 'jacobians.f', 'sigTrials.f', 'r.m', 'initStateVars.f', 'ElasticMatrix.f');
    
 fortran2(Res,      'File', [folder, '\residuals.f']);
 fortran2(Jacobis,  'File', [folder, '\jacobians.f']);
 fortran2(sigTrial, 'File', [folder, '\sigTrials.f']);
 fortran2(v00,      'File', [folder, '\initStateVars.f']);
 fortran2(D,        'File', [folder, '\ElasticMatrix.f']);
 
 materialGeneration(ModelName, matrixProp, c, v, v0, Res)
disp('Finished')
toc


%% Output Matlab files (for simple strain driver)
%h = {v,v0,c,de};

%disp('Generating Matlab functions..');
%tic

% Output
%delete('r.m', 'ja.m','s.m','f.m', 'v00.m');
%matlabFunction(R,'File','r.m','Vars',h);
%matlabFunction(J,'File','ja.m','Vars',h);
%matlabFunction(S,'File','s.m','Vars',h);
%matlabFunction(F,'File','f.m','Vars',h);
%matlabFunction(v00,'File','v00.m','Vars',h);

%toc
