module precision
  ! Dictionary:
  !    SGL - Single precision
  !    DBL - Double precision
  !-----------------------------------------------------------------------------------------

  ! Set prescision for Real
  !-----------------------------------------------------------------------------------------
  integer, parameter :: realSGL = selected_real_kind (p = 6,  r = 37)
  integer, parameter :: realDBL = selected_real_kind (p = 15, r = 307)

  ! Set prescision for Integer
  !-----------------------------------------------------------------------------------------
  integer, parameter :: intSGL = selected_real_kind (p = 9)
  integer, parameter :: intDBL = selected_real_kind (p = 18)

  ! Set prescision for Complex
  !-----------------------------------------------------------------------------------------
  integer, parameter :: complexSGL = realSGL
  integer, parameter :: complexDBL = realDBL

  !integer, parameter :: complexKindSingle = selected_real_kind (p = 6,  r = 37)
  !integer, parameter :: complexKindDouble = selected_real_kind (p = 15, r = 307)
end module precision
!-------------------------------------------------------------------------------
!-------------------------------------------------------------------------------
module standardVars ! This module contains a list of minimum needed variables.
  use precision
  real(kind=realDBL), dimension(6) :: dEpsilon, Sig
  real(kind=realDBL), dimension(7) :: minStVars, minStVars0
  real(kind=realDBL) :: de11,de12,de13,de22,de23,de33
  real(kind=realDBL) ::  s11, s12, s13, s22, s23, s33
  real(kind=realDBL) :: s110,s120,s130,s220,s230,s330
  real(kind=realDBL) ::  lam, lam0
end module standardVars
!-------------------------------------------------------------------------------
!-------------------------------------------------------------------------------
module mapping
  use precision
  use standardVars
  implicit none
  private
!-------------------------------------------------------------------------------
    type, public :: matlabSymbolics
    contains
      procedure :: numhelp          => numhelp
      procedure :: standardMapping  => standardMapping
      procedure :: hyd              => hyd
      procedure :: dev              => dev
    end type matlabSymbolics
!-------------------------------------------------------------------------------
  contains
    subroutine numhelp(this)
    ! The purpose of this method is to ensure nonzero J2.
      class(matlabSymbolics) :: this
      real(kind=realDBL) :: hydStress, devStress
      real(kind=realDBL) :: err
      call this%standardMapping()

      err = 1.0D-6

      Sig(1) = s11
      Sig(2) = s22
      Sig(3) = s33
      Sig(4) = s12
      Sig(5) = s23
      Sig(6) = s13

      hydStress = this%hyd(Sig,6)
      devStress = this%dev(Sig,6)

      if (devStress>-err .or. devStress<err) then
        s22  = s22  + hydStress*err
        s220 = s220 + hydStress*err
      end if
    end subroutine numhelp
!-------------------------------------------------------------------------------
    subroutine standardMapping(this)
      class(matlabSymbolics) :: this
      ! Purpose:
      !   The subroutine maps variables used in the code to the defined variables
      !   in "user vars" (i.e. global variables).
      ! Minimum new stateVars
      s11  = minStVars(1)
      s22  = minStVars(2)
      s33  = minStVars(3)
      s12  = minStVars(4)
      s23  = minStVars(5)
      s13  = minStVars(6)
      lam  = minStVars(7)
      ! Minimum old stateVars
      s110 = minStVars0(1)
      s220 = minStVars0(2)
      s330 = minStVars0(3)
      s120 = minStVars0(4)
      s230 = minStVars0(5)
      s130 = minStVars0(6)
      lam0 = minStVars0(7)
      ! Strain increment
      de11 = dEpsilon(1)
      de22 = dEpsilon(2)
      de33 = dEpsilon(3)
      de12 = dEpsilon(4)
      de23 = dEpsilon(5)
      de13 = dEpsilon(6)
    end subroutine standardMapping
!-------------------------------------------------------------------------------
  function hyd(this, vec, n) result(y)
    class(matlabSymbolics) :: this
    integer :: n
    real(kind=realDBL) :: y
    real(kind=realDBL), dimension(n) :: vec
    y = 1.0D0/3.0D0*(vec(1)+vec(2)+vec(3))
  end function hyd
!-------------------------------------------------------------------------------
  function dev(this, vec, n) result(y)
    class(matlabSymbolics) :: this
    integer :: n
    real(kind=realDBL) :: p, y
    real(kind=realDBL), dimension(n) :: vec
    p = this%hyd(vec, n)
    y = sqrt(3.D0/2.D0*( &
      (vec(1)-p)**2.D0 + (vec(2)-p)**2.D0 + (vec(3)-p)**2.D0 &
      + 2.D0 * (vec(4)**2.D0 + vec(5)**2.D0 + vec(6)**2.D0)))
  end function dev
!-------------------------------------------------------------------------------
!-------------------------------------------------------------------------------
end module mapping
!-------------------------------------------------------------------------------
!-------------------------------------------------------------------------------
module matlabCalculations
  use precision
  !-----------------------------------------------------------------------------
  ! Purpose:
  !   The following functions are used by the Matlab generated Fortran code.
  !-----------------------------------------------------------------------------
    interface asin
      module procedure asin1
    end interface

    interface acos
      module procedure acos1
    end interface

    interface conjg
      module procedure conjg1
    end interface

    interface heaviside
      module procedure heaviside1!, heaviside2
    end interface

    interface dirac
      module procedure dirac1
    end interface
!-------------------------------------------------------------------------------
!-------------------------------------------------------------------------------
  contains
!-------------------------------------------------------------------------------
    function asin1(a) result(u)
      real(kind=realDBL), intent(in) :: a
      real(kind=realDBL) :: u
      real(kind=realDBL) :: temp
      temp = a
      u = asin(temp)
    end function
!-------------------------------------------------------------------------------
    function acos1(a) result(u)
      real(kind=realDBL), intent(in) :: a
      real(kind=realDBL) :: u
      real(kind=realDBL) :: temp
      temp = a
      u = acos(temp)
    end function
!-------------------------------------------------------------------------------
     function conjg1(a) result(u)
       real(kind=realDBL) :: u, a
       u = a
     end function
!-------------------------------------------------------------------------------
    function heaviside1(xx) result(u)
       real(kind=realDBL) :: xx, u
       if (xx > 0.0) then
        u = 1.0d0
      else if (xx == 0.0) then
        u = 0.5d0
      else
        u = 0.0d0
      end if
    end function
!-------------------------------------------------------------------------------
    function heaviside2(xx) result(u)
       real(kind=realDBL) :: xx
       real(kind=realDBL) :: u, temp
       temp = xx
       u = heaviside1(temp)
    end function
!-------------------------------------------------------------------------------
    function dirac1(x) result(y)
        real(kind=realDBL) x,y
        y = 0.0d0
    end function
!-------------------------------------------------------------------------------
!-------------------------------------------------------------------------------
end module matlabCalculations
