import sys
import csv

from simple_project import SimpleProject, run 

class ExportCSVProject(SimpleProject):
    def add_soil_layers(self):
        
        self._g_input.SoilContour.initializerectangular(0, 0, 100, 30)
        
        borehole = self._g_input.borehole(0)
        self._g_input.soillayer(0)
        self._g_input.setsoillayerlevel(borehole, 0, 30)
        self._g_input.setsoillayerlevel(borehole, 1, 27)
        self._g_input.soillayer(0)
        self._g_input.setsoillayerlevel(borehole, 2, 15)
        self._g_input.soillayer(0)
        self._g_input.setsoillayerlevel(borehole, 3, 0)
        borehole.Head = 23
        
    def add_structures(self):
        self._g_input.gotostructures()
        self._g_input.plate(40, 30, 40, 14)
        self._g_input.plate(60, 30, 60, 14)
        
        for plate in self._g_input.Plates:
            self._g_input.posinterface(plate.Parent)
            self._g_input.neginterface(plate.Parent)
            
        self._g_input.lineload(28, 30, 38, 30)
        
    def select_curve_points(self):
        output_port = self._g_input.selectmeshpoints()
        s_output, g_output = self._new_server('localhost', output_port)
        g_output.addcurvepoint('node', 40, 30, -1, 0)
        g_output.update()
        
    def configure_phases(self):
        self._g_input.gotostages()
        
        self._g_input.activate(self._g_input.LineLoads, self._g_input.InitialPhase)
        self._g_input.activate(self._g_input.Plates, self._g_input.InitialPhase)
        self._g_input.activate(self._g_input.Interfaces, self._g_input.InitialPhase)
        self._g_input.deactivate(self._g_input.SoilPolygons[1], self._g_input.InitialPhase)
        
        line_load = self._g_input.LineLoads[0]
        
        current_phase = self._g_input.InitialPhase
        line_load.qy_start[current_phase] = 0.0
        
        for i in range(1, 10):
            current_phase = self._g_input.phase(current_phase)
            self._g_input.setcurrentphase(current_phase)
            line_load.qy_start[current_phase] = -10.0 * i
        
        
    def gather_results(self):
        output_port = self._g_input.view(self._g_input.InitialPhase)
        s_output, g_output = self._new_server('localhost', output_port)
        
        self._results = []
        
        # phaseID is only correct because we know the phases are in order!
        for phaseID, phase in enumerate(g_output.Phases):
            if phaseID == 0:
                # Skip the initial phase and just write 0, because getcurveresults won't return any results for it
                self._results.append({'phaseID': 0, 'displacement': 0})
            else:
                displacement_string = g_output.getcurveresults(g_output.Nodes[0], phase, g_output.ResultTypes.Soil.Utot)
                self._results.append({'phaseID': phaseID, 'displacement': displacement_string})
                
        g_output.update()
        
    def output_results(self):
        #Prevents excess newlines
        if sys.version_info.major == 2:
            file_kwargs = {'mode': 'wb'}
        else:
            file_kwargs = {'mode': 'w', 'newline': ''}
            
        # Bt default the csv writers will separate values with a comma and the
        # period will be used as the decimal separator.
        # However, this might be problematic when trying to import a csv file
        # on a system that uses comma as the decimal separator.
        # In these cases do the following:
        # - import the locale modules and call: locale.setlocale(locale.LC_ALL, '')
        # - get the correct separator with: locale.localeconv()['decimal_point']
        # - replace the period in the output results with the correct separator
        # - use a different csv dialect (see Python docs) so the comma isn't used
        # as the column separator.
        with open("results.csv", **file_kwargs) as csv_file:
            writer = csv.DictWriter(csv_file, fieldnames=['phaseID', 'displacement'])
            for row in self._results:
                writer.writerow(row)
                
if __name__ == '__main__':
    run(ExportCSVProject)