import os
import imp
import subprocess

class BlankSlate(object):
    """
    Class that provides a way to quickly setup a project in PLAXIS.
    Based on PLAXIS example.
    """
    
    def __init__(self, plaxisPath, serverPort):
        """
        The init function imports the boilercode from PLAXIS 2D and sets up
        an interface between the Python script and PLAXIS program.
        
        The interface is handled by two object instances:
            GUI instance (for creating new/loading projects etc.)
            Model instance (size, material, structures, mesh, etc.)
        
        Input parameters are the plaxis directory path and the server port.
        """
        
        # Import module from plaxis_path. This enables us to store this script anywhere we want to.
        found_module = imp.find_module('plxscripting', [plaxisPath])
        plxscripting = imp.load_module('plxscripting', *found_module)
        
        from plxscripting.easy import new_server
        self._new_server = new_server
        
        # Starts PLAXIS.exe
        args = [os.path.join(plaxisPath, "Plaxis2DXInput.exe"), "--appServerPort={}".format(serverPort)]
        self._input_process = subprocess.Popen(args)
        
        #subprocess.Popen(['C:\\Program Files (x86)\\Plaxis\\PLAXIS 2D\\Plaxis2dXInput.exe', "--AppServerPort="+str(serverPort)])
        
        # Creates instances of the PLAXIS GUI and Model
        self.GUI_input, self.model = self._new_server('localhost', serverPort, timeout=10.0)
        
        self.GUI_input.new()
    
    
    def gather_results(self):
        """
        This function needs to be defined in a child object.
        The purpose of this function is to gather data in output view
        
        Examples are PLAXIS plots, curveresults and values at Nodes/Gauss points
        """
        raise NotImplementedError("Override gather_results in subclass.")
        
        
    def output_results(self):
        """
        This function needs to be defined in a child object.
        The purpose of this function is to output the gathered or calculated 
            data from gathered values
        
        Examples are saving CSV, plots and showing figures.
        """
        raise NotImplementedError("Override output_results in subclass.")
        
        
    def close_input(self):
        """
        This functions terminates the script and PLAXIS.
        """
        self._input_process.kill()
        
    # @property
    # def m_Input(self):
    #     """
    #     Creates a reference to self name: self.m_Input.
    #     """
    #     return self.m_Input
     
        
    def add_soil_layers(self):
        """
        This function needs to be defined in a child object.
        The purpose of this function is to define the soil layers in the model. 
            
        Multiple boreholes, soil layers and water head may be defined.
        """
        raise NotImplementedError("Override add_soil_layers in subclass.")
      
        
    def apply_soil_material(self):
        """
        Default soil material that is applied to all soil layers.
        
        The function may be overwritten or changed directly in blankSlate.py.
        """
        einit = 0
        ninit = einit/(1+einit)
        Material = [
        ('MaterialName', 'PLAXIS Cam-Clay'),
        ('Colour', 10676870),
        ('SoilModel', 8),
        ('DrainageType', 'Undrained A'),
        ('gammaUnsat', 20),('gammaSat', 20),
        ('lambdaModified', 0.25/(1+einit)),
        ('kappaModified', 0.05/(1+einit)),
        ('ninit', ninit),
        ('phi', 25.4),
        ('nu', 0.15),
        ('M', 1),
        ("InterfaceStrength", 1),
        ("Gref", 1),
        ("cref", 1) ,
        # ('K0Primary', 1),
        # ('K0Secondary',0.5),
        ('OCR', 4),
        ('POP', 0.0)]


        clay = self.model.soilmat(*Material)
    
        for soil_layer in self.model.SoilLayers:
            self.model.setmaterial(soil_layer, clay)
       
        
    def add_structures(self):
        """
        The purpose of this function is to create default structures (plate,
            load, interface, etc).
            
        At the moment the function adds nothing to the model.
        
        The function may be overwritten or changed directly in blankSlate.py.
        """
        pass # Not adding any plates is fine too.
       
        
    def apply_plate_material(self):
        """
        Default plate material that is applied to all plate in the model.
        
        The function may be overwritten or changed directly in blankSlate.py.
        """
        DIAPHRAGM_WALL_PARAMETERS = [
        ('MaterialName', 'Wall'),
        ('Colour', 16711680),
        ('Elasticity', 0),
        ('IsIsotropic', True),
        ('IsEndBearing', True),
        ('EA', 12000000),
        ('EI', 120000),
        ('nu', 0.15),
        ('d', 0.34641),
        ('w', 8.3),
        ('Mp', 1000000000000000.0),
        ('Np', 10000000000.0),
        ('Np2', 10000000000.0),
        ('RayleighAlpha', 0),
        ('RayleighBeta', 0),
        ('Gref', 15061311)
        ]
        diaphragm_wall_material = self.model.platemat(*DIAPHRAGM_WALL_PARAMETERS)
        
        for plate in self.model.Plates:
            self.model.setmaterial(plate, diaphragm_wall_material)
            
            
    def mesh(self):
        """
        Default meshing value is used at 0.06.
        
        The function may be overwritten or changed directly in blankSlate.py.
        """
        self.model.gotomesh() 
        self.model.mesh(0.06)
        
    def select_curve_points(self):
        """
        Default selection of Nodes/Gauss Points for curve results.
        
        At the moment none points are selected.
        
        The function may be overwritten or changed directly in blankSlate.py.
        """
        pass # Not selecting any curve-points is fine too.
        
        
    def configure_phases(selef):
        """
        This function needs to be defined in a child object.
        The purpose of this function is to define the phases (model history). 
            
        Examples are activating/deactivating soil clusters, adding load to
            different phases, changing groundwater level, creating new phase 
            branch, changing materials and etc.
        """
        raise NotImplementedError("Override configure_phases in subclass.")
        
    def make_project(self):
        """
        This function creates the entire model and calculates the result.
            
        The build up is:
            1) Define soil layers
            2) Add soil materials to layers
            3) Define structures
            4) Add plate material to plates (or structures)
            5) Mesh the model
            6) Select Nodes/Gauss points
            7) Configure phases 
            8) Start calculation 
        """
        self.add_soil_layers()
        self.apply_soil_material()
        self.add_structures()
        self.apply_plate_material()
        self.mesh()
        self.select_curve_points()
        self.configure_phases()
        self.model.calculate()
        
def run(PLAXIS_projectClass, plaxisPath = r"C:\Program Files (x86)\Plaxis\PLAXIS 2D",  INPUT_SERVER_PORT=10000):
    """
    This function creates a instance of the given PLAXIS project class.
    It defines the model as is given in make_project() and it gathers and 
        output the result as defined in gather_results() and close_input()
    """
    project = PLAXIS_projectClass(plaxisPath, INPUT_SERVER_PORT)
    project.make_project()
    project.gather_results()
    project.output_results()
    #project.close_input()
  
if __name__ == '__main__':
    run(BlankSlate)