import sys, os
import numpy as np
from matplotlib import pyplot

from blankSlate import BlankSlate, run 

try:
    from PIL import Image, ImageFont, ImageDraw
except ImportError:
    raise Exception("Pillow must be installed to use this script.")

class ElemTriaxialTest(BlankSlate):
    def add_soil_layers(self):
        mm = 1/1000
        
        self.model.SoilContour.initializerectangular(0, 0, 27*mm, 50*mm)
        
        borehole = self.model.borehole(0)
        self.model.soillayer(50*mm)
        borehole.Head = -50**mm
               
        self.model.setproperties('UnitLength', "m", "ModelType", "Axisymmetry", 'UnitForce', "kN")
        
        
    def add_structures(self):
        mm = 1/1000
        
        #self.model.gotostructures()
        self.model.lineload(0, 0, 27*mm, 0)
        self.model.lineload(27*mm, 0, 27*mm, -50*mm)
        
        # For strain controlled shearing
        self.model.linedispl(self.model.lines[0])
        self.model.linedispl(self.model.lines[1])
        
        # For boundaries
        self.model.linedispl(0, 0, 0, -50*mm)
        self.model.linedispl(0, -50*mm, 27*mm, -50*mm)
        
        
    def mesh(self):
        self.model.gotomesh()
        self.model.Lines[0].CoarsenessFactor = 1
        self.model.mesh(0.05)
   
    def select_curve_points(self):
        mm = 1/1000
         
        output_port = self.model.selectmeshpoints()
        s_output, g_output = self._new_server('localhost', output_port)
        g_output.addcurvepoint('stresspoint', 13.5*mm, -25*mm)
        g_output.update()
    
    def configure_phases(self):
        self.model.gotostages()
        
        ## K0 - calculation
        initial_phase = self.model.Phases[0]
        
        # free, normallyfixed, horizontallyfixed, verticallyfixed, fullyfixed
        self.model.Deformations.BoundaryXMin[initial_phase] = 'horizontallyfixed'
        self.model.Deformations.BoundaryXMax[initial_phase] = 0
        self.model.Deformations.BoundaryYMin[initial_phase] = 'verticallyfixed'
        self.model.Deformations.BoundaryYMax[initial_phase] = 0
        
        self.model.activate(self.model.LineDisplacements, initial_phase)
        self.model.LineDisplacements[1].Displacement_x[initial_phase] = "fixed"
        self.model.LineDisplacements[1].Displacement_y[initial_phase] = "free"
        self.model.LineDisplacements[3].Displacement_x[initial_phase] = "free"
        self.model.LineDisplacements[3].Displacement_y[initial_phase] = "fixed"
        
        self.model.deactivate(self.model.LineDisplacements[0], self.model.LineDisplacements[2], initial_phase)
        
        ## Consolidation
        cons_phase    = self.model.phase(initial_phase)
        self.model.Phases[1].Identification = "Consolidation"
        self.model.Phases[1].TimeInterval = 1
        self.model.Phases[1].Deform.IgnoreUndrainedBehaviour = True
        
        self.model.activate(  self.model.LineLoads, cons_phase)
        self.model.LineLoads[0].qy_start[cons_phase] = -100
        self.model.LineLoads[1].qy_start[cons_phase] =    0
        self.model.LineLoads[1].qx_start[cons_phase] = -100
        
        ## Strain controlled shearing
        strainControlled = 0
        epsilon = 10/100 # %
        mm = 1/1000
        
        ## Active shear
        active_phase  = self.model.phase(cons_phase)
        self.model.Phases[2].Identification = "Active shear"
        self.model.Phases[2].TimeInterval = 1
        
        if strainControlled == 1:
            self.model.activate(self.model.LineDisplacements[0], active_phase)
            self.model.deactivate(self.model.LineLoads, self.model.LineDisplacements[2], active_phase)
            self.model.LineDisplacements[0].uy_start[active_phase] = -50*mm*epsilon
            self.model.LineDisplacements[0].Displacement_x[active_phase] = "Free"
        else:
            self.model.LineLoads[0].qy_start[active_phase] = -500
        
        ## Passive shear
        passive_phase  = self.model.phase(cons_phase)
        self.model.Phases[3].Identification = "Passive shear"
        self.model.Phases[3].TimeInterval = 1
        
        if strainControlled == 1:
            self.model.activate(self.model.LineDisplacements[2], passive_phase)
            self.model.deactivate(self.model.LineLoads, self.model.LineDisplacements[0], passive_phase)
            self.model.LineDisplacements[2].Displacement_y[passive_phase] = "Free"
            self.model.LineDisplacements[2].Displacement_x[passive_phase] = "Prescribed"
            self.model.LineDisplacements[2].ux_start[passive_phase] = -50*mm*epsilon
        else:
            self.model.LineLoads[1].qx_start[passive_phase] = -500
        
        # # Numerical control parameters
        # self.model.Phases[1].Deform.UseDefaultIterationParams = False
        # self.model.Phases[1].Deform.MaxUnloadingSteps = 10
    

    def gather_results(self):
        width, height = 800, 600
        
        output_port = self.model.view(self.model.Phase_1)
        s_output, g_output = self._new_server('localhost', output_port)
        
        # Hide everything except the mesh itself.
        switch = False
        
        g_output.Plots[0].DrawFrame = switch
        g_output.Plots[0].DrawTitle = switch
        g_output.Plots[0].DrawLegend = switch
        g_output.Plots[0].DrawAxes = switch
  
        self._image = []
        for i in range(1,4):
            g_output.Plots[0].Phase = g_output.Phases[i]
            g_output.set(g_output.Plots[0].ResultType, g_output.soil.Utot)
            self._image.append(g_output.Plots[0].export(width, height))
            
            g_output.set(g_output.Plots[0].ResultType, g_output.soil.IncrementalDeviatoricStrain)
            self._image.append(g_output.Plots[0].export(width, height))
        
            g_output.set(g_output.Plots[0].ResultType, g_output.soil.PExcess)
            self._image.append(g_output.Plots[0].export(width, height))
        
   #        #   g_output.set(g_output.Plots[0].ResultType, g_output.soil.MeanEffStress)
            # self._image.append(g_output.Plots[0].export(width, height))
        
   #        #   g_output.set(g_output.Plots[0].ResultType, g_output.soil.DeviatoricStress)
            # self._image.append(g_output.Plots[0].export(width, height))
        
        
        activeDevStress  = g_output.getcurveresultspath(g_output.StressPoints[0], g_output.Phases[1], g_output.Phases[2], g_output.ResultTypes.Soil.DeviatoricStress)
        passiveDevStress = g_output.getcurveresultspath(g_output.StressPoints[0], g_output.Phases[1], g_output.Phases[3], g_output.ResultTypes.Soil.DeviatoricStress)
        
        activeDevStrain  = g_output.getcurveresultspath(g_output.StressPoints[0], g_output.Phases[1], g_output.Phases[2], g_output.ResultTypes.Soil.TotalDeviatoricStrain)
        passiveDevStrain = g_output.getcurveresultspath(g_output.StressPoints[0], g_output.Phases[1], g_output.Phases[3], g_output.ResultTypes.Soil.TotalDeviatoricStrain)
        
        self._results = [activeDevStress.echo(), activeDevStrain.echo(), passiveDevStress.echo(), passiveDevStrain.echo()]
        
        g_output.update()
        
    def output_results(self):
        #textTitles = ['Utot', 'Incremental Deviatoric Strain', 'Pexcess', 'Mean Eff. Stress', 'Deviatoric Stress']
        textTitles = ['Utot', 'Incremental Deviatoric Strain', 'Pexcess']
        phases     = ['Consolidation', 'Active test', 'Passive test']
        j          = 0
        
        for number, img in enumerate(self._image):
            # Not an elegant solution...
            if number < len(textTitles):
                i = 0
            elif number >= len(textTitles) and number <len(textTitles)*2:
                i = 1
            else:
                i = 2

            if number == len(textTitles):
                j = 0
            elif number == len(textTitles)*2:
                j = 0
            
            figure = img.image
        
            result_image = Image.new(figure.mode, figure.size)
            result_image.paste(figure)
        
            default_font = ImageFont.load_default()
            drawing_context = ImageDraw.Draw(result_image)
            text = phases[i] +' - '+ textTitles[j]
            drawing_context.text((figure.size[0]/2-50,figure.size[1]-50), text, font=default_font, fill=(0,0,0,255))
            
            if not os.path.exists('./figures'):
                os.makedirs('./figures')
            
            result_image.save("./figures\\"+ text +".png")
        
            j+=1
            
        
        stressStrain = []
        for data in self._results:
            floatValues = np.array(data[1:-2].split(','))
            floatValues = floatValues.astype('float')
            stressStrain.append(floatValues)
            
        activeDevStress  = stressStrain[0]
        activeDevStrain  = stressStrain[1]
        
        passiveDevStress = stressStrain[2]
        passiveDevStrain = stressStrain[3]
            
        pyplot.plot(activeDevStrain,   activeDevStress, 'r')
        pyplot.plot(passiveDevStrain, -passiveDevStress, 'b')
        pyplot.grid(True)
        pyplot.show()
        
if __name__ == '__main__':
    run(ElemTriaxialTest)